/*
	$Id: ParentInputMode.cpp,v 1.17 2003/05/14 12:03:14 phonohawk Exp $
	---------
	
    MacOS X implementation of the SKK input method.
    Copyright (C) 2002 phonohawk

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "ParentInputMode.h"
#include "ChildInputMode.h"
#include "BIM.h"
#include <assert.h>

ParentInputMode::ParentInputMode()
	:hiragana_input_mode(*this),
	 zen_kata_input_mode(*this),
	 han_kata_input_mode(*this),
	 han_asc_input_mode(*this),
	 zen_asc_input_mode(*this),
	 kanji_conversion_mode(*this),
	 ascii_convertion_mode(*this),
	 is_kanji_mode(false),
	 is_ascii_conv_mode(false),
	 current_input_mode(&hiragana_input_mode)
{
	
}

ParentInputMode::~ParentInputMode()
{

}

bool ParentInputMode::handleInput(unsigned char inputChar)
{
	ChildInputMode* input_mode;
	if (is_kanji_mode)
	{
		input_mode = &kanji_conversion_mode;
	}
	else if (is_ascii_conv_mode)
	{
		input_mode = &ascii_convertion_mode;
	}
	else
	{
		if (inputChar >= 'A' && inputChar <= 'Z' && inputChar != 'L' &&	//XXX
			!isHanAscInputMode() && !isZenAscInputMode())
		{
			is_kanji_mode = true;
			input_mode = &kanji_conversion_mode;
		}
		else
		{
			input_mode = current_input_mode;
		}
	}
	assert(input_mode != NULL);
	
	bool do_fix = input_mode->handleInput(inputChar);
	if (do_fix)
	{
		fixUnlessIgnore(input_mode->getStringToFix());
	}
	displayUnlessIgnore(input_mode->getStringToDisplay());
	
	return false; // falseԂBʂɈӖ͖B
}

bool ParentInputMode::handleBackSpace()
{
	ChildInputMode& input_mode = getReallyCurrentInputMode();
	bool handled = input_mode.handleBackSpace();
	
	if (handled)
	{
		// ωB
		displayUnlessIgnore(input_mode.getStringToDisplay());
	}
	
	return handled;
}

bool ParentInputMode::handleReturn()
{
	ChildInputMode& input_mode = getReallyCurrentInputMode();
	
	bool handled = input_mode.handleReturn();
	if (!handled)
	{
		fixUnlessIgnore(input_mode.getStringToFix());
	}
	displayUnlessIgnore(input_mode.getStringToDisplay());
	return handled;
}

bool ParentInputMode::handleArrow(int kind)
{
	ChildInputMode& input_mode = getReallyCurrentInputMode();
	
	bool handled = input_mode.handleArrow(kind);
	if (!handled)
	{
		fixUnlessIgnore(input_mode.getStringToFix());
	}
	displayUnlessIgnore(input_mode.getStringToDisplay());
	return handled;
}

bool ParentInputMode::handleCg()
{
	bool handled = getReallyCurrentInputMode().handleCg();
	displayUnlessIgnore(getReallyCurrentInputMode().getStringToDisplay());
	return handled;
}

bool ParentInputMode::handleCy()
{
	bool handled = getReallyCurrentInputMode().handleCy();
	displayUnlessIgnore(getReallyCurrentInputMode().getStringToDisplay());
	return handled;
}

bool ParentInputMode::handleTab()
{
	bool handled = getReallyCurrentInputMode().handleTab();
	displayUnlessIgnore(getReallyCurrentInputMode().getStringToDisplay());
	return handled;
}

void ParentInputMode::redisplay()
{
	CppCFString str_to_display = getReallyCurrentInputMode().getStringToDisplay();
	if (str_to_display.length() == 0 ||
		(str_to_display.length() > 0 && str_to_display[0] != kIgnoreThis)) // kIgnoreThisȂXVȂB
	{
		display(str_to_display);
	}
}

void ParentInputMode::updatePencilMenuIcon()
{
	if (current_input_mode == dynamic_cast<ChildInputMode*>(&hiragana_input_mode)) {
		BIMSetPencilMenuIcon(kHiraganaIconIndex);
	}
	else if (current_input_mode == dynamic_cast<ChildInputMode*>(&zen_kata_input_mode)) {
		BIMSetPencilMenuIcon(kZenKataIconIndex);
	}
	else if (current_input_mode == dynamic_cast<ChildInputMode*>(&han_kata_input_mode)) {
		BIMSetPencilMenuIcon(kHanKataIconIndex);
	}
	else if (current_input_mode == dynamic_cast<ChildInputMode*>(&han_asc_input_mode)) {
		BIMSetPencilMenuIcon(kHanAscIconIndex);
	}
	else if (current_input_mode == dynamic_cast<ChildInputMode*>(&zen_asc_input_mode)) {
		BIMSetPencilMenuIcon(kZenAscIconIndex);
	}
}

void ParentInputMode::terminate()
{
	ChildInputMode& input_mode = getReallyCurrentInputMode();
	
	input_mode.terminate();
	fixUnlessIgnore(input_mode.getStringToFix());
	redisplay();
}

void ParentInputMode::displayUnlessIgnore(const CppCFString& str)
{
	if (str.length() == 0 ||
		(str.length() > 0 && str[0] != kIgnoreThis)) // kIgnoreThisȂfixȂB
	{
		display(str);
	}
}

void ParentInputMode::fixUnlessIgnore(const CppCFString& str)
{
	if (str.length() == 0 ||
		(str.length() > 0 && str[0] != kIgnoreThis)) // kIgnoreThisȂfixȂB
	{
		fix(str);
	}
}

ChildInputMode& ParentInputMode::getCurrentInputMode()
{
	return *current_input_mode;
}

ChildInputMode& ParentInputMode::getReallyCurrentInputMode()
{
	if (is_kanji_mode)
	{
		return kanji_conversion_mode;
	}
	else if (is_ascii_conv_mode)
	{
		return ascii_convertion_mode;
	}
	else
	{
		return *current_input_mode;
	}
}

HiraganaInputMode& ParentInputMode::getHiraganaInputMode()
{
	return hiragana_input_mode;
}

KanjiConversionMode& ParentInputMode::getKanjiConversionMode()
{
	return kanji_conversion_mode;
}

void ParentInputMode::goHiraganaInputMode()
{
	current_input_mode = &hiragana_input_mode;
	updatePencilMenuIcon();
}

void ParentInputMode::goZenKataInputMode()
{
	current_input_mode = &zen_kata_input_mode;
	updatePencilMenuIcon();
}

void ParentInputMode::goHanKataInputMode()
{
	current_input_mode = &han_kata_input_mode;
	updatePencilMenuIcon();
}

void ParentInputMode::goHanAscInputMode()
{
	current_input_mode = &han_asc_input_mode;
	updatePencilMenuIcon();
}

void ParentInputMode::goZenAscInputMode()
{
	current_input_mode = &zen_asc_input_mode;
	updatePencilMenuIcon();
}

bool ParentInputMode::isHiraganaInputMode() const
{
	return current_input_mode == dynamic_cast<const ChildInputMode*>(&hiragana_input_mode);
}

bool ParentInputMode::isZenKataInputMode() const
{
	return current_input_mode == dynamic_cast<const ChildInputMode*>(&zen_kata_input_mode);
}

bool ParentInputMode::isHanKataInputMode() const
{
	return current_input_mode == dynamic_cast<const ChildInputMode*>(&han_kata_input_mode);
}

bool ParentInputMode::isHanAscInputMode() const
{
	return current_input_mode == dynamic_cast<const ChildInputMode*>(&han_asc_input_mode);
}

bool ParentInputMode::isZenAscInputMode() const
{
	return current_input_mode == dynamic_cast<const ChildInputMode*>(&zen_asc_input_mode);
}

void ParentInputMode::setEnabledKanjiMode(bool enabled)
{
	is_kanji_mode = enabled;
}

void ParentInputMode::setEnabledAsciiConvertionMode(bool enabled)
{
	is_ascii_conv_mode = enabled;
	
	if (enabled)
	{
		ascii_convertion_mode.start();
		redisplay();
	}
}
